% This script calculates the unfactored M-N curve for an unreinforced or reinforced (rebar or I-beam) concrete liner (approximate solution)
% Compression is POSITIVE in this calculation
clear;
clc;
tic;

%% USER INPUTS

% Solution Information
etmax = -0.01; % Maximum tensile strain: -0.01 (default)

% Beam Geometry
beam_b = 1; % Beam length (in m) (1m default)
beam_h = 0.3; % Beam thickness (in m) - thickness of shotcrete

% Concrete Properties
conc_f1c = 45; % Concrete compressive strength (in MPa)
conc_f0 = 1; % Concrete tensile strength (in MPa): 0 for plain concrete, <0 for fibre-reinforced
conc_E1 = 4500*sqrt(conc_f1c); % Concrete elastic modulus (in MPa)

% Steel Properties (FOR TYPE = 2 OR 3)
steel_fy = 400; % Steel strength (in MPa)
steel_E1 = 200000; % Steel elastic modulus (in MPa)

% Reinforcement Properties
rein_type = 2; % Type of reinforcement: 1 for concrete only, 2 for rebar, 3 for I-beams

    % Rebar (FOR TYPE = 2) - assumes symetric reinforcement (same # of rows of rebar at top and bottom)
    rebar_rows = 1; % Number of rows of rebar at top or bottom
    rebar_spacing = 0.10; % Rebar spacing along beam length (in m)
    rebar_diam = 0.0252; % Rebar diameter (in m)
    rebar_cover = 0.040; % Clear cover to face of rebar (in m)
    rebar_clear = 0; % Clear spacing between rows of rebar (in m) - only needed if rebar_rows > 1

    % I-beam (FOR TYPE = 3)
    ibeam_spacing = 1; % I-beam spacing along beam length (in m)
    ibeam_h = 0.140; % I-beam height (in m)
    ibeam_flangew = 0.073; % Flange width (in m)
    ibeam_flanget = 0.0069; % Flange thickness (in m)
    ibeam_webt = 0.0047; % Web thickness (in m)

%% INITIALIZE VARIABLES BASED ON INPUTS

conc_ec = (0.0146*conc_f1c+1.5182)/1000; % (conc_f1c in MPa) Peak compressive concrete strain
e1c = 0.0032; % Maximum strain at the extreme concrete compression fibre (ultimate strain for concrete)

conc_f0 = -abs(conc_f0); % Ensures tensile strength is negative number

A_beam = beam_b*beam_h; % Area of beam (in m^2)

if rein_type == 1 % Concrete only
    A_steel = 0; % Area of steel (in m^2)
    
elseif rein_type == 2 % Rebar reinforcement
    bal_et = -steel_fy/steel_E1; % Tensile strain at balanced condition
    
    % Area calculations
    rebar_num = beam_b/rebar_spacing; % Number of rebars per row along beam length
    A_rebar = pi*(rebar_diam/2)^2; % Area of a single rebar (in m^2)
    A_rebarrow = rebar_num*A_rebar; % Area of row of rebar (in m^2)
    rebar_totalrow = 2*rebar_rows; % Total number of rows of rebar
    A_steel = rebar_totalrow*A_rebarrow; % Total area of rebar in beam (in m^2)
    
    % Reinforcement locations
    rein_d = zeros(rebar_rows*2);
    for i = 1:rebar_rows % Define x locations for each rebar row (at midpoint of rebar row, so cover + diameter)
        if i == 1
            rein_d(i) = rebar_cover+rebar_diam/2; % Depth to midpoint of upper rebar row
        else
            rein_d(i) = rein_d(i-1)+rebar_clear+rebar_diam; % Depth to midpoint of second rebar row (if present)
        end
        rein_d(2*rebar_rows-i+1) = beam_h-rein_d(i); % Depth to midpoint of lower rebar row
    end
    
elseif rein_type == 3 % I-beam reinforcement
    bal_et = -steel_fy/steel_E1; % Tensile strain at balanced condition
    
    % Area calculations
    ibeam_num = beam_b/ibeam_spacing; % Number of I-beams along beam length
    ibeam_webw = ibeam_h-2*ibeam_flanget; % Height of I-beam web (m)
    A_ibeam = 2*(ibeam_flangew*ibeam_flanget)+ibeam_webw*ibeam_webt; % Cross-sectional area of beam (in m^2)
    ibeam_cover = (beam_h-ibeam_h)/2; % Clear cover to top of I-beam (in m)
    ibeam_flangetot = ibeam_flangew*ibeam_num; % Total flange width for beam (in m)
    ibeam_webtot = ibeam_webt*ibeam_num; % Total web width for beam (in m)
    A_steel = A_ibeam*ibeam_num; % Area of I-beam in beam (in m^2)
    
    % Reinforcement Zones
    rein_d1 = ibeam_cover; % Distance to top of I-beam (in m)
    rein_d2 = ibeam_cover+ibeam_flanget; % Distance to bottom of top flange (in m)
    rein_d3 = ibeam_cover+ibeam_h-ibeam_flanget; % Distance to top of bottom flange (in m)
    rein_d4 = ibeam_cover+ibeam_h; % Distance to bottom of I-beam (in m)
    
end

A_conc = A_beam-A_steel; % Area of concrete in beam (in m^2)

%% COMBINED AXIAL & BENDING

% How many sections to divide the beam into (used to approximate integration)
sections = 1000;

% Establish vector of strains to sample (each strain is a point along the M-N curve
einc = -0.0001; % Strain increment - defines how many points are generated on M-N curve

ec_range = e1c+einc:einc:0; % Compressive strain (top of beam)
et_range = e1c:einc:etmax; % Tensile strain (bottom of beam)

q = length(ec_range);
r = length(et_range);

% Initialize output matrix
MNcurve1 = zeros(r+q,2); % Points on M-N diagram (right side of y axis)
MNcurve2 = zeros(r+q,2); % Points on M-N diagram (left side of y axis)

% Solve for axial force and bending moment for each strain
for i = 1:(r+q) % Step through each strain value
    
    if i <= r % Establishes ec and et values for each point on the M-N curve (value of i)
        ec = e1c;
        et = et_range(i);
    else
        ec = ec_range(i-r);
    end
    
    strain = @(x) (et-ec)/beam_h.*x+ec; % Strain for a given depth x along the beam
    
    % CONCRETE LOADS
    
    conc_cumforce = 0;
    conc_cummoment = 0;
    
    for j = 1:sections % Calculate force and moment for each section of the beam (simulates integration over beam thickness)
        
        x1 = (j-1)*(beam_h/sections); % Distance to top of section (in m)
        x2 = j*(beam_h/sections); % Distance to bottom of section (in m)
        xm = (x1+x2)/2; % Midpoint of section (in m)
        
        conc_stress1 = ConcStress_Unfact(x1,beam_h,et,ec,conc_ec,conc_f1c,conc_f0); % Stress at top of section (in MPa)
        conc_stress2 = ConcStress_Unfact(x2,beam_h,et,ec,conc_ec,conc_f1c,conc_f0); % Stress at bottom of section (in MPa)
        conc_stress = (conc_stress1+conc_stress2)/2; % Average stress for section (in MPa)
        
        conc_force = conc_stress*beam_b*(beam_h/sections); % Factored concrete force (in MN)
        conc_moment = conc_force*(beam_h/2-xm); % Factored concrete moment (in MNm)
        
        conc_cumforce = conc_cumforce + conc_force; % Cumulative concrete force (in MN)
        conc_cummoment = conc_cummoment + conc_moment; % Cumulative concrete moment (in MNm)
        
    end
    
    % REINFORCEMENT LOADS
    
    if rein_type == 1 % Unreinforced Concrete
        
        rein_cumforce = 0; % Cumulative reinforcement force (in MN)
        rein_cummoment = 0; % Cumulative reinforcement moment (in MNm)
        
    elseif rein_type == 2 % Rebar Reinforcement
        
        rein_cumforce = 0;
        rein_cummoment = 0;
        
        for j = 1:rebar_totalrow % Calculate force and moment at each rebar row
            
            x = rein_d(j); % Distance to rebar row (in m)
            rein_strain = strain(x); % Strain at rebar row
            
            % Steel stress at rebar locations (in MPa)
            steel_stress = SteelStress(x,et,ec,beam_h,steel_E1,steel_fy);
            
            % Concrete stress at rebar locations (subtract from steel stress) (in MPa)
            cs_stress = ConcStress_Unfact(x,beam_h,et,ec,conc_ec,conc_f1c,conc_f0);
            
            rein_force = (steel_stress-cs_stress)*A_rebarrow; % Reinforcement force (in MN)
            rein_moment = rein_force*(beam_h/2-x); % Reinforcement moment (in MNm)
            
            rein_cumforce = rein_cumforce + rein_force; % Cumulative reinforcement force (in MN)
            rein_cummoment = rein_cummoment + rein_moment; % Cumulative reinforcement moment (in MNm)
            
        end
        
    elseif rein_type == 3 % I-beam Reinforcement
        
        rein_cumforce = 0;
        rein_cummoment = 0;
        
        for j = 1:sections/100 % Calculate force and moment for each section of the I-beam upper flange (simulates integration over flange thickness)
            
            x1 = rein_d1+(ibeam_flanget/10)*(j-1); % Distance to top of section (in m)
            x2 = rein_d1+(ibeam_flanget/10)*j; % Distance to bottom of section (in m)
            xm = (x1+x2)/2; % Midpoint of section (in m)
            
            steel_stress1 = SteelStress(x1,et,ec,beam_h,steel_E1,steel_fy); % Steel stress at top of section (in MPa)
            cs_stress1 = ConcStress_Unfact(x1,beam_h,et,ec,conc_ec,conc_f1c,conc_f0); % Concrete stress at top of section (in MPa)
            rein_stress1 = steel_stress1-cs_stress1; % Reinforcement stress at top of section (in MPa)
            steel_stress2 = SteelStress(x2,et,ec,beam_h,steel_E1,steel_fy); % Steel stress at bottom of section (in MPa)
            cs_stress2 = ConcStress_Unfact(x2,beam_h,et,ec,conc_ec,conc_f1c,conc_f0); % Concrete stress at bottom of section (in MPa)
            rein_stress2 = steel_stress2-cs_stress2; % Reinforcement stress at bottom of section (in MPa)
            rein_stress = (rein_stress1+rein_stress2)/2; % Average stress for section (in MPa)
            
            rein_force = rein_stress*ibeam_flangetot*(ibeam_flanget/10); % Reinforcement force (in MN)
            rein_moment = rein_force*(beam_h/2-xm); % Reinforcement moment (in MNm)
            
            rein_cumforce = rein_cumforce + rein_force; % Cumulative reinforcement force (in MN)
            rein_cummoment = rein_cummoment + rein_moment; % Cumulative reinforcement moment (in MNm)
            
        end
        
        for k = 1:sections/10 % Calculate force and moment for each section of the I-beam web (simulates integration over web width)
            
            x1 = rein_d2+(ibeam_webw/100)*(k-1); % Distance to top of section (in m)
            x2 = rein_d2+(ibeam_webw/100)*k; % Distance to bottom of section (in m)
            xm = (x1+x2)/2; % Midpoint of section (in m)
            
            steel_stress1 = SteelStress(x1,et,ec,beam_h,steel_E1,steel_fy); % Steel stress at top of section (in MPa)
            cs_stress1 = ConcStress_Unfact(x1,beam_h,et,ec,conc_ec,conc_f1c,conc_f0); % Concrete stress at top of section (in MPa)
            rein_stress1 = steel_stress1-cs_stress1; % Reinforcement stress at top of section (in MPa)
            steel_stress2 = SteelStress(x2,et,ec,beam_h,steel_E1,steel_fy); % Steel stress at bottom of section (in MPa)
            cs_stress2 = ConcStress_Unfact(x2,beam_h,et,ec,conc_ec,conc_f1c,conc_f0); % Concrete stress at bottom of section (in MPa)
            rein_stress2 = steel_stress2-cs_stress2; % Reinforcement stress at bottom of section (in MPa)
            rein_stress = (rein_stress1+rein_stress2)/2; % Average stress for section (in MPa)
            
            rein_force = rein_stress*ibeam_webtot*(ibeam_webw/100); % Reinforcement force (in MN)
            rein_moment = rein_force*(beam_h/2-xm); % Reinforcement moment (in MNm)
            
            rein_cumforce = rein_cumforce + rein_force; % Cumulative reinforcement force (in MN)
            rein_cummoment = rein_cummoment + rein_moment; % Cumulative reinforcement moment (in MNm)
            
        end
        
        for l = 1:sections/100 % Calculate force and moment for each section of the I-beam lower flange (simulates integration over flange thickness)
            
            x1 = rein_d3+(ibeam_flanget/10)*(l-1); % Distance to top of section (in m)
            x2 = rein_d3+(ibeam_flanget/10)*l; % Distance to bottom of section (in m)
            xm = (x1+x2)/2; % Midpoint of section (in m)
            
            steel_stress1 = SteelStress(x1,et,ec,beam_h,steel_E1,steel_fy); % Steel stress at top of section (in MPa)
            cs_stress1 = ConcStress_Unfact(x1,beam_h,et,ec,conc_ec,conc_f1c,conc_f0); % Concrete stress at top of section (in MPa)
            rein_stress1 = steel_stress1-cs_stress1; % Reinforcement stress at top of section (in MPa)
            steel_stress2 = SteelStress(x2,et,ec,beam_h,steel_E1,steel_fy); % Steel stress at bottom of section (in MPa)
            cs_stress2 = ConcStress_Unfact(x2,beam_h,et,ec,conc_ec,conc_f1c,conc_f0); % Concrete stress at bottom of section (in MPa)
            rein_stress2 = steel_stress2-cs_stress2; % Reinforcement stress at bottom of section (in MPa)
            rein_stress = (rein_stress1+rein_stress2)/2; % Average stress for section (in MPa)
            
            rein_force = rein_stress*ibeam_flangetot*(ibeam_flanget/10); % Reinforcement force (in MN)
            rein_moment = rein_force*(beam_h/2-xm); % Reinforcement moment (in MNm)
            
            rein_cumforce = rein_cumforce + rein_force; % Cumulative reinforcement force (in MN)
            rein_cummoment = rein_cummoment + rein_moment; % Cumulative reinforcement moment (in MNm)
            
        end
        
    end
    
    Moment = (conc_cummoment + rein_cummoment); % Moment for given value of eb (in MNm)
    Axial = (conc_cumforce + rein_cumforce); % Axial force for given value of eb (in MN)
    
    MNcurve1(i,1:2) = [Moment Axial];
    MNcurve2((q+r)-i+1,1:2) = [-Moment Axial];
    
end

MNcurve = MNcurve1; % MN Curve (right side only)
MNcurveFull = [MNcurve1; MNcurve2]; % MN Curve (right and left sides)

figure(1)
beam_h = plot(MNcurveFull(:,1),MNcurveFull(:,2),'Color','r','LineWidth',3);
grid on
title('Capacity Plot','FontName','Arial','FontSize',14)
xlabel('Bending Moment (MNm)','FontName','Arial','FontSize',12)
ylabel('Axial Force (MN)','FontName','Arial','FontSize',12)
saveas(gcf, 'MNCapacityPlot_Unfactored_Approx', 'jpg');

toc;